/***********************************************************************************
Projekt:	Wetterstation

Autoren:	Benjamin Drner 	MM5 165941
	     	Jrgen Hasselbach	MM5 165944

Datum:		28.01.08

***********************************************************************************/
#include "sht11.h"




// assembler inline fr nop


// SHT11 ROUTINE

//----------------------------------------------------------------------------------
char s_write_byte(unsigned char value)
//----------------------------------------------------------------------------------
// writes a byte on the Sensibus and checks the acknowledge
{
	unsigned char i,error=0;
	PORTA |= (1<<PA2);
	DDRA  |= (1<<DDA2);
	for (i=0x80;i>0;i/=2) //shift bit for masking
	{ 
		if (i & value) 
			PORTA |= (1<<DATA);  //masking value with i , write to SENSI-BUS
		else 
			PORTA &= ~(1<<DATA);
		PORTA |= (1<<SCK);   //SCK=1; //clk for SENSI-BUS
		NOP;NOP;NOP;         //pulswith approx. 5 us
		NOP;NOP;NOP;         
		NOP;NOP;NOP;
	    NOP;NOP;NOP;         
		NOP;NOP;NOP;
		NOP;NOP;NOP;         
	    NOP;NOP;NOP;
		NOP;NOP;NOP; 					        
		PORTA &= ~(1<<SCK);  //SCK=0;
	}
	
//	PORTA |= (1<<DATA); //DATA=1; //release DATA-line
    DDRA  &= ~(1<<DDA2);
	PORTA |= (1<<SCK); 	//SCK=1; //clk #9 for ack
	error = PINA & (1<<DATA);    //check ack (DATA will be pulled down by SHT11)
	PORTA &= ~(1<<SCK); //SCK=0;
	return error; 		//error=1 in case of no acknowledge
}
//----------------------------------------------------------------------------------
char s_read_byte(unsigned char ack)
//----------------------------------------------------------------------------------
// reads a byte form the Sensibus and gives an acknowledge in case of "ack=1"
{
  unsigned char i,val=0;
  //PORTA |=  (1<<PA2);
  DDRA  &= ~(1<<DDA2);

//  PORTA |= (1<<DATA);          // DATA=1; //release DATA-line
  for (i=0x80;i>0;i/=2)        // shift bit for masking
  { 
	PORTA |= (1<<SCK);         // SCK=1; //clk for SENSI-BUS
	if (PINA & (1<<DATA)) 
	  val=(val | i);           // read bit
	PORTA &= ~(1<<SCK);        // SCK=0;
  }

  PORTA &= (!ack<<DATA);       // DATA=!ack; //in case of "ack==1" pull down DATA-Line
  PORTA |= (1<<SCK);           // SCK=1; //clk #9 for ack
  NOP;NOP;NOP;                 // pulswith approx. 5 us
  PORTA &= ~(1<<SCK);          // SCK=0;
  PORTA |= (1<<DATA);          // DATA=1; //release DATA-line
  return val;
}
//----------------------------------------------------------------------------------
void s_transstart(void)
//----------------------------------------------------------------------------------
// generates a transmission start
//       _____         ________
// DATA:      |_______|
//           ___     ___
// SCK : ___|   |___|   |______
{

	PORTA |= (1<<DATA);
    DDRA  |= (1<<DDA2);
	PORTA &= ~(1<<SCK);// DATA=1; SCK=0; //Initial state
	NOP;
	PORTA |=  (1<<SCK);  // SCK=1;
	NOP;
	PORTA &= ~(1<<DATA); // DATA=0;
	NOP;
	PORTA &= ~(1<<SCK);  // SCK=0;
	NOP;NOP;NOP;
	PORTA |=  (1<<SCK);  // SCK=1;
	NOP;
	PORTA |=  (1<<DATA); // DATA=1;
	NOP;
	PORTA &= ~(1<<SCK);  // SCK=0;
}
//----------------------------------------------------------------------------------
void s_connectionreset(void)
//----------------------------------------------------------------------------------
// communication reset: DATA-line=1 and at least 9 SCK cycles followed by transstart
//       _____________________________________________________         ________
// DATA:                                                      |_______|
//          _    _    _    _    _    _    _    _    _        ___     ___
// SCK : __| |__| |__| |__| |__| |__| |__| |__| |__| |______|   |___|   |______
{
	unsigned char i;
	PORTA |= (1<<DATA);
	PORTA &= ~(1<<SCK);// DATA=1; SCK=0; //Initial state
	for(i=0;i<9;i++) //9 SCK cycles
	{ 
		PORTA |= (1<<SCK);  // SCK=1;
		PORTA &= ~(1<<SCK); // SCK=0;
	}
	s_transstart(); //transmission start
}
//----------------------------------------------------------------------------------
char s_softreset(void)
//----------------------------------------------------------------------------------
// resets the sensor by a softreset
{
	unsigned char error=0;
	s_connectionreset(); //reset communication
	error+=s_write_byte(RESET); //send RESET-command to sensor
	return error; //error=1 in case of no response form the sensor
}
//----------------------------------------------------------------------------------
char s_read_statusreg(unsigned char *p_value, unsigned char *p_checksum)
//----------------------------------------------------------------------------------
// reads the status register with checksum (8-bit)
{
	unsigned char error=0;
	s_transstart(); //transmission start
	error=s_write_byte(STATUS_REG_R); //send command to sensor
	*p_value=s_read_byte(ACK); //read status register (8-bit)
	*p_checksum=s_read_byte(noACK); //read checksum (8-bit)
	return error; //error=1 in case of no response form the sensor
}
//----------------------------------------------------------------------------------
char s_write_statusreg(unsigned char *p_value)
//----------------------------------------------------------------------------------
// writes the status register with checksum (8-bit)
{
	unsigned char error=0;
	s_transstart(); //transmission start
	error+=s_write_byte(STATUS_REG_W);//send command to sensor
	error+=s_write_byte(*p_value); //send value of status register
	return error; //error>=1 in case of no response form the sensor
}
//----------------------------------------------------------------------------------
char s_measure(unsigned char *p_value, unsigned char *p_checksum, unsigned char mode)
//----------------------------------------------------------------------------------
// makes a measurement (humidity/temperature) with checksum
{
	unsigned error=0;
	unsigned int i;
	s_transstart(); //transmission start
	switch(mode)	//send command to sensor
	{ 
		case TEMP : 
		  error+=s_write_byte(MEASURE_TEMP); 
		break;
		case HUMI : 
		  error+=s_write_byte(MEASURE_HUMI); 
		break;
		
		default : 
		break;
	}
	for (i=0;i<65535;i++)
	{ 
	  if((PINA & (1<<DATA)) == 0) 
	    break; //wait until sensor has finished the measurement
	}
	if(PINA & (1<<DATA)) 
	  error+=1;                       // or timeout (~2 sec.) is reached
	*(p_value)  = s_read_byte(ACK);   // read the first byte (MSB)
	*(p_value+1)= s_read_byte(ACK);   // read the second byte (LSB)
	*p_checksum = s_read_byte(noACK); // read checksum
	return error;
}

//----------------------------------------------------------------------------------------
void calc_sth11(float *p_humidity ,float *p_temperature)
//----------------------------------------------------------------------------------------
// calculates temperature [C] and humidity [%RH]
// input : humi [Ticks] (12 bit)
// temp [Ticks] (14 bit)
// output: humi [%RH]
// temp [C]
{
  const float C1=-4.0;       // for 12 Bit
  const float C2= 0.0405;    // for 12 Bit
  const float C3=-0.0000028; // for 12 Bit
  const float T1=0.01;       // for 14 Bit @ 5V
  const float T2=0.00008;    // for 14 Bit @ 5V
  float rh=*p_humidity;      // rh: Humidity [Ticks] 12 Bit
  float t=*p_temperature;    // t: Temperature [Ticks] 14 Bit
  float rh_lin;              // rh_lin: Humidity linear
  float rh_true;             // rh_true: Temperature compensated humidity
  float t_C;                 // t_C : Temperature [C]
  t_C=t*0.01 - 40.1;           // calc. Temperature from ticks to [C]
  rh_lin=C3*rh*rh + C2*rh + C1; //calc. Humidity from ticks to [%RH]
  rh_true=(t_C-25)*(T1+T2*rh)+rh_lin; //calc. Temperature compensated humidity [%RH]
  if(rh_true>100)
    rh_true=100;            // cut if the value is outside of
  if(rh_true<0.1)
    rh_true=0.1;            // the physical possible range
  *p_temperature=t_C;       // return temperature [C]
  *p_humidity=rh_true;      // return humidity[%RH]
}

//--------------------------------------------------------------------
float calc_dewpoint(float h,float t)
//--------------------------------------------------------------------
// calculates dew point
// input: humidity [%RH], temperature [C]
// output: dew point [C]
{ 
	float k,dew_point ;
	k = (log10(h)-2)/0.4343 + (17.62*t)/(243.12+t);
	dew_point = 243.12*k/(17.62-k);
	return dew_point;
}
