#include "common.h"
#include "user_interface.h"
#include "serial.h"
#include <stdlib.h>
#include <string.h>
#include <assert.h>


///////////////////////////////////////////////////////////////////////////////
// @function :

void print_str(char_t* ps_string)
//
// @abstract: print a string
//
// @param   : pi_string - the actual string to print 
//                        \0 terminated, max 64 Bytes 
//
// @return  : -
//
// @history :
//
// name   date        action
// ----------------------------------------------------------------------------
// ode    16.07.2009  implementation start
///////////////////////////////////////////////////////////////////////////////

{
  uint8_t strLen = 0;

  while(   (*ps_string != '\0') 
        && (strLen <= 64))
  {
    USART_SendByte(*ps_string);
	  ps_string++;
	  strLen++;
  } 
}
// 
///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
// @function :

void print_int(uint8_t pi_format, uint32_t pi_number, ws_type_t ws_type, uint8_t fieldwidth, align_t align)
//
// @abstract: print an integer
//
// @param   : pi_format - print integer number format 2 = binary, 
//                                                   10 = decimal
//                                                   16 = hex
//                        (see itoa() function for legal values)
//            pi_number - the actual value to be printed
//            ws_type   - the type of 
//
// @return  : 
//
// @history :
//
// name   date        action
// ----------------------------------------------------------------------------
// ode    16.07.2009  implementation start
///////////////////////////////////////////////////////////////////////////////
{
  char_t  str[65];
  uint8_t len;
  uint8_t spcToPrint;
  uint8_t cnt;

  assert(fieldwidth < 65);

	itoa(pi_number, str, pi_format);
  len = strlen(str);

  if (fieldwidth > len)
  {
    spcToPrint = fieldwidth - len;
    switch (align)
    {
      case L_ALIGN :
        for (cnt = 0; cnt < spcToPrint; spcToPrint++)
        {
          USART_SendByte(' ');
        }
        print_str(str);
      break;

      case R_ALIGN :
        print_str(str);
        for (cnt = 0; cnt < spcToPrint; spcToPrint++)
        {
          USART_SendByte(' ');
        }
      break;

      case C_ALIGN :
        for (cnt = 0; cnt < spcToPrint/2; spcToPrint++)
        {
          USART_SendByte(' ');
        }
        print_str(str);
        for (cnt = 0; cnt < (spcToPrint - (spcToPrint/2)); spcToPrint++)
        {
          USART_SendByte(' ');
        }
      break;

      case V_ALIGN :  // fieldwidth and align are ingored ...
      default:

      break;
    }
  }




  switch(ws_type)
  {
    case CR:
    case CRLF:
    case SPC:
    case VOID:
    {
    }
    break;

    case OxCR:
    case OxCRLF:
    case OxSPC:
    case OxVOID:
    {
      USART_SendByte('0');
      USART_SendByte('x');
    }
    break;
  }

  print_str(str);
  
  switch(ws_type)
  {
    case CR:
    case OxCR:
    {
      USART_SendByte('\n');
    }
    break;

    case CRLF:
    case OxCRLF:
    {
      USART_SendByte('\n');
      USART_SendByte('\r');
    }
    break;

    case SPC:
    case OxSPC:
     {
      USART_SendByte(' ');
    }
    break;

    default:  // VOID or 0xVOID etc. ignore ...
      ;
  }
}
 

