/***********************************************************************************
Projekt:	Wetterstation

Autoren:	Benjamin Drner 	MM5 165941
	     	Jrgen Hasselbach	MM5 165944

Datum:		28.01.08

***********************************************************************************/
#ifndef F_CPU
#define F_CPU 18432000UL //Quarz-Frequency
#endif


//#include <intrins.h> 		//Keil library (is used for _nop()_ operation)
#include <avr/io.h> 		//header-file for I/O configurations
#include <math.h> 			//Keil library
#include <stdio.h> 			//Keil library
#include <avr/interrupt.h> 	//header-file for global interrupt activation
#include <stdint.h> 		//header-file zur Definition der Zahlentypen (int,char, unsigned, char usw.)
#include <stdbool.h> 		//header-Datei zur Definition der 1-Bit-Variablenbool
#include <util\delay.h> 	//header-Datei zur Definition frWarteschleifen/zeiten
#include <E:\Wetterstation\##eigenes##\AVR\lcd_lib_steffen.c>//header-file for lcd display
#include <E:\Wetterstation\##eigenes##\AVR\float.h>

typedef union
{ 
	unsigned int i;
	float f;
} value;

//inline
#define NOP asm volatile ("nop") // assembler inline fr nop

//VARIABLEN
unsigned char prescaler10ms;
bool b_tick10ms;				// flag for 10ms tick
unsigned char anzeige=0;		//flag to switch display;
//variablen fr ####mpx4115A####
char 	result_LCD[5];
char 	voltage_LCD[5];
char 	druck_LCD[4];
int 	i;
float 	result;
float 	voltage;
float 	druck;
float	druck_alt;
float	druck_neu;
float	fhumi_val;
float	ftemp_val;
int		ihumi_val;
int		itemp_val;

//KONSTANTEN
#define PRESCALER_TICK_10MS 90 //Prescaler tick for 10ms
//konstanten fr ####sht11####
enum {TEMP,HUMI};
#define DATA PD2
#define SCK PD3
#define noACK 0
#define ACK 1

void DisplayNumber(unsigned int number, unsigned char row, unsigned char col, unsigned char length);


//adr command r/w
#define STATUS_REG_W 0x06 		//000 0011 0
#define STATUS_REG_R 0x07 		//000 0011 1
#define MEASURE_TEMP 0x03 		//000 0001 1
#define MEASURE_HUMI 0x05 		//000 0010 1
#define RESET 0x1e 				//000 1111 0

//Timer 0 initialisation in normal mode

void InitTimer0()

{

TCCR0A |=(0<<WGM01) | (0<<WGM00);
TCCR0B |= (1<<CS01); 				// Vorteiler 8 aus Tabelle
TIMSK0 |= (1<<TOIE0);
}


// Interrupt routine for timer0

ISR(TIMER0_OVF_vect)
{
	--prescaler10ms; 							// Decrease prescaler10ms
	if(prescaler10ms == 0)
	{
		b_tick10ms = true; 						//set Flag 10ms tick
		prescaler10ms = PRESCALER_TICK_10MS; 	//load constant
	}
}


//Function for reading buttons

inline uint8_t debounce (volatile uint8_t *port,uint8_t pin)
{
	if(!(*port & (1<<pin)))
	{
		_delay_ms(100); 		// Pin put to ground
		if((*port & (1<<pin)))
		{
			_delay_ms(100); 	// Time for user to put up finger
			return pin;
		}
	}
	return 0;
}

// Button control function
void ButtonControl()
{

	// Startknopf S1
	if (debounce(&PINC, PC0))
	{
		++anzeige;			//inkrementiere b_anzeige
		if(anzeige>2)
		{
			anzeige=0;
		}
		
	}

}



// SHT11 ROUTINE



//----------------------------------------------------------------------------------
char s_write_byte(unsigned char value)
//----------------------------------------------------------------------------------
// writes a byte on the Sensibus and checks the acknowledge
{
	unsigned char i,error=0;
	PORTD |= (1<<PD2);
	DDRD |= (1<<DDD2);
	for (i=0x80;i>0;i/=2) //shift bit for masking
	{ 
		if (i & value) 
			PORTD &= (1<<DATA); //masking value with i , write to SENSI-BUS
		else 
			PORTD &= ~(1<<DATA);
			PORTD &= (1<<SCK); //SCK=1; //clk for SENSI-BUS
			NOP;NOP;NOP; //pulswith approx. 5 us
			PORTD &= ~(1<<SCK); //SCK=0;
	}
	
	PORTD &= (1<<DATA); //DATA=1; //release DATA-line
	PORTD &= (1<<SCK); 	//SCK=1; //clk #9 for ack
	error=DATA; 		//check ack (DATA will be pulled down by SHT11)
	PORTD &= ~(1<<SCK); //SCK=0;
	return error; 		//error=1 in case of no acknowledge
}
//----------------------------------------------------------------------------------
char s_read_byte(unsigned char ack)
//----------------------------------------------------------------------------------
// reads a byte form the Sensibus and gives an acknowledge in case of "ack=1"
{
	unsigned char i,val=0;
	PORTD |= (1<<PD2);
	DDRD |= ~(1<<DDD2);

	PORTD &= (1<<DATA); // DATA=1; //release DATA-line
	for (i=0x80;i>0;i/=2) //shift bit for masking
	{ 
		PORTD &= (1<<SCK); //SCK=1; //clk for SENSI-BUS
		if (DATA) val=(val | i); //read bit
		PORTD &= ~(1<<SCK); //SCK=0;
	}

	PORTD &= (!ack<<DATA); //DATA=!ack; //in case of "ack==1" pull down DATA-Line
	PORTD &= (1<<SCK); //SCK=1; //clk #9 for ack
	NOP;NOP;NOP; //pulswith approx. 5 us
	PORTD &= ~(1<<SCK); //SCK=0;
	PORTD &= (1<<DATA); //DATA=1; //release DATA-line
	return val;
}
//----------------------------------------------------------------------------------
void s_transstart(void)
//----------------------------------------------------------------------------------
// generates a transmission start
// _____ ________
// DATA: |_______|
// ___ ___
// SCK : ___| |___| |______
{
	PORTD &= (1<<DATA)|~(1<<SCK);//DATA=1; SCK=0; //Initial state
	NOP;
	PORTD &= (1<<SCK); //SCK=1;
	NOP;
	PORTD &= ~(1<<DATA); //DATA=0;
	NOP;
	PORTD &= ~(1<<SCK); //SCK=0;
	NOP;NOP;NOP;
	PORTD &= (1<<SCK); //SCK=1;
	NOP;
	PORTD &= (1<<DATA); //DATA=1;
	NOP;
	PORTD &= ~(1<<SCK); //SCK=0;
}
//----------------------------------------------------------------------------------
void s_connectionreset(void)
//----------------------------------------------------------------------------------
// communication reset: DATA-line=1 and at least 9 SCK cycles followed by transstart
// _____________________________________________________ ________
// DATA: |_______|
// _ _ _ _ _ _ _ _ _ ___ ___
// SCK : __| |__| |__| |__| |__| |__| |__| |__| |__| |______| |___| |______
{
	unsigned char i;
	PORTD &= (1<<DATA)|~(1<<SCK); //DATA=1; SCK=0; //Initial state
	for(i=0;i<9;i++) //9 SCK cycles
	{ 
		PORTD &= (1<<SCK); //SCK=1;
		PORTD &= ~(1<<SCK); //SCK=0;
	}
	s_transstart(); //transmission start
}
//----------------------------------------------------------------------------------
char s_softreset(void)
//----------------------------------------------------------------------------------
// resets the sensor by a softreset
{
	unsigned char error=0;
	s_connectionreset(); //reset communication
	error+=s_write_byte(RESET); //send RESET-command to sensor
	return error; //error=1 in case of no response form the sensor
}
//----------------------------------------------------------------------------------
char s_read_statusreg(unsigned char *p_value, unsigned char *p_checksum)
//----------------------------------------------------------------------------------
// reads the status register with checksum (8-bit)
{
	unsigned char error=0;
	s_transstart(); //transmission start
	error=s_write_byte(STATUS_REG_R); //send command to sensor
	*p_value=s_read_byte(ACK); //read status register (8-bit)
	*p_checksum=s_read_byte(noACK); //read checksum (8-bit)
	return error; //error=1 in case of no response form the sensor
}
//----------------------------------------------------------------------------------
char s_write_statusreg(unsigned char *p_value)
//----------------------------------------------------------------------------------
// writes the status register with checksum (8-bit)
{
	unsigned char error=0;
	s_transstart(); //transmission start
	error+=s_write_byte(STATUS_REG_W);//send command to sensor
	error+=s_write_byte(*p_value); //send value of status register
	return error; //error>=1 in case of no response form the sensor
}
//----------------------------------------------------------------------------------
char s_measure(unsigned char *p_value, unsigned char *p_checksum, unsigned char mode)
//----------------------------------------------------------------------------------
// makes a measurement (humidity/temperature) with checksum
{
	unsigned error=0;
	unsigned int i;
	s_transstart(); //transmission start
	switch(mode)	//send command to sensor
	{ 
		case TEMP : error+=s_write_byte(MEASURE_TEMP); break;
		case HUMI : error+=s_write_byte(MEASURE_HUMI); break;
		default : break;
	}
	for (i=0;i<65535;i++) if(DATA==0) break; //wait until sensor has finished the measurement
	if(DATA) error+=1; // or timeout (~2 sec.) is reached
	*(p_value) =s_read_byte(ACK); //read the first byte (MSB)
	*(p_value+1)=s_read_byte(ACK); //read the second byte (LSB)
	*p_checksum =s_read_byte(noACK); //read checksum
	return error;
}

//----------------------------------------------------------------------------------------
void calc_sth11(float *p_humidity ,float *p_temperature)
//----------------------------------------------------------------------------------------
// calculates temperature [C] and humidity [%RH]
// input : humi [Ticks] (12 bit)
// temp [Ticks] (14 bit)
// output: humi [%RH]
// temp [C]
{
	const float C1=-4.0; // for 12 Bit
	const float C2= 0.0405; // for 12 Bit
	const float C3=-0.0000028; // for 12 Bit
	const float T1=0.01; // for 14 Bit @ 5V
	const float T2=0.00008; // for 14 Bit @ 5V
	float rh=*p_humidity; // rh: Humidity [Ticks] 12 Bit
	float t=*p_temperature; // t: Temperature [Ticks] 14 Bit
	float rh_lin; // rh_lin: Humidity linear
	float rh_true; // rh_true: Temperature compensated humidity
	float t_C; // t_C : Temperature [C]
	t_C=t*0.01 - 40; //calc. Temperature from ticks to [C]
	rh_lin=C3*rh*rh + C2*rh + C1; //calc. Humidity from ticks to [%RH]
	rh_true=(t_C-25)*(T1+T2*rh)+rh_lin; //calc. Temperature compensated humidity [%RH]
	if(rh_true>100)rh_true=100; //cut if the value is outside of
	if(rh_true<0.1)rh_true=0.1; //the physical possible range
	*p_temperature=t_C; //return temperature [C]
	*p_humidity=rh_true; //return humidity[%RH]
}

//--------------------------------------------------------------------
float calc_dewpoint(float h,float t)
//--------------------------------------------------------------------
// calculates dew point
// input: humidity [%RH], temperature [C]
// output: dew point [C]
{ 
	float k,dew_point ;
	k = (log10(h)-2)/0.4343 + (17.62*t)/(243.12+t);
	dew_point = 243.12*k/(17.62-k);
	return dew_point;
}

// mpx4115A routine


//	Spannung_einlesen: 	liest die Spannung am Analogeingang des ADC ein			   


void Spannung_einlesen(void)
{
	result = 0;
	i = 0;
																// ADC aktivieren und
	ADCSRA |= (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);	// Frequenzvorteiler auf 128 setzen (144 kHz)

	ADMUX = 5;										// PC5 als Eingang der Spannung vom MPX4115A gewaelt
	
	ADMUX |= (0<<ADLAR) | (1<<REFS0) | (0<<REFS1); 	// externe Referenzspannung AVcc (+5V) nutzen 

	ADCSRA |= (1<<ADSC);							// eine ADC-Wandlung (Warmlauf)

	while (ADCSRA & (1<<ADSC));						// auf Abschluss der Konvertierung warten

	for(i=0;i<10;i++)								// FOR-Schleife fuer Mittelwertbildung
	{
		ADCSRA |= (1<<ADSC);						// eine ADC-Wandlung
		while (ADCSRA & (1<<ADSC));					// auf Abschluss der Konvertierung warten
		result += ADCW;								// 10 Wandlungen aufsummieren
	}

	ADCSRA &= ~(1<<ADEN);							// ADC deaktivieren (spart Strom)

	result /= 10;									// Mittelwertbildung
}


//	Umrechnung_Volt:	rechnet den eingelesenen Wert in Volt um				   


void Umrechnung_Volt(void)
{
	voltage = 0;
	voltage = result * 4.93 / 1023;
}



//	Umrechnung_mbar:	rechnet die anliegende Spannung in mbar um				


void Umrechnung_mbar(void)
{
	druck = 0;
	druck = ((voltage / 4.93) + 0.095) / 0.009;
}

//________________________________________________


//LCD Routine

void lcd_writer()
{
	// enable Display
	DDRC = 0x0f;
	DDRD = 0xff;
	PORTC = 0x0f;

	switch(anzeige)
	{
		case 0:	lcd_init();
				lcd_clearline(255);
				lcd_gotoxy(0,0);
				lcd_putstr("TEMP\0");
				DisplayNumber(ftemp_val,0,6,6);
				lcd_gotoxy(0,13);
				lcd_putstr("C\0");
				lcd_gotoxy(1,0);
				lcd_putstr("HUMI\0");
				DisplayNumber(fhumi_val,1,6,6);
				lcd_gotoxy(1,13);
				lcd_putstr("%\0");

		case 1: lcd_init();
				lcd_clearline(255);
				lcd_gotoxy(0,0);
				lcd_putstr("HUMI\0");
				DisplayNumber(fhumi_val,0,6,6);
				lcd_gotoxy(0,13);
				lcd_putstr("%\0");
				lcd_gotoxy(1,0);
				lcd_putstr("PRES\0");
				DisplayNumber(druck,1,6,4);
				lcd_gotoxy(1,11);
				lcd_putstr("mbar\0");

		case 2: lcd_init();
				lcd_clearline(255);
				lcd_gotoxy(0,0);
				lcd_putstr("PRES\0");
				DisplayNumber(druck,0,6,4);
				lcd_gotoxy(0,11);
				lcd_putstr("mbar\0");
				lcd_gotoxy(1,0);
				lcd_putstr("TEMP\0");
				DisplayNumber(ftemp_val,1,6,6);
				lcd_gotoxy(1,13);
				lcd_putstr("C\0");

	}

	// disable Dispaly, enable Buttons
	DDRC = 0x00;
}



void DisplayNumber(unsigned int number, unsigned char row, unsigned char col, unsigned char length)
{
	unsigned char i=0;
	unsigned char j=0;
	unsigned char tempnumber[10] = {0,0,0,0,0,0,0,0,0,0};

	while (number>0) // divides number in one digit
	{
		++i;
		tempnumber[i]=number%10;
		number = number / 10;
	}

		if (i<length)
		i=length;

		for (j=length ; j>0; j--)
		{
			lcd_gotoxy
			(row,col-j); // write digits on correct position

			switch (tempnumber[i])
			{
				case 0: lcd_putstr ("0\0");
				break;
				case 1: lcd_putstr ("1\0");
				break;
				case 2: lcd_putstr ("2\0");
				break;
				case 3: lcd_putstr ("3\0");
				break;
				case 4: lcd_putstr ("4\0");
				break;
				case 5: lcd_putstr ("5\0");
				break;
				case 6: lcd_putstr ("6\0");
				break;
				case 7: lcd_putstr ("7\0");
				break;
				case 8: lcd_putstr ("8\0");
				break;
				case 9: lcd_putstr ("9\0");
				break;
			}

		--i;
		}
}


//---------------------------------------------------------------------------------

/*
int main (void)
{




	sei();
	InitTimer0();
	lcd_init();
	prescaler10ms = PRESCALER_TICK_10MS;

	while(1)
	{
		ButtonControl();
	
		value humi_val,temp_val;
		float dew_point;
		unsigned char error,checksum;
		unsigned int i;
		s_connectionreset();
		if(b_tick10ms)
		{ 	error=0;
			error+=s_measure((unsigned char*) &humi_val.i,&checksum,HUMI); //measure humidity
			error+=s_measure((unsigned char*) &temp_val.i,&checksum,TEMP); //measure temperature
			if(error!=0) s_connectionreset(); //in case of an error: connection reset
			
			else			
			{ 
				fhumi_val=(float)ihumi_val; //converts integer to float
				ftemp_val=(float)itemp_val; //converts integer to float
				calc_sth11(&fhumi_val,&ftemp_val); //calculate humidity, temperature
				//dew_point=calc_dewpoint(humi_val.f,temp_val.f); //calculate dew point
				//send final data to serial interface (UART)
				//printf("temp:%5.1fC humi:%5.1f%% dew point:%5.1fC\n",temp_val.f,humi_val.f,dew_point);
				//lcd_write();
			}

			Spannung_einlesen();
			Umrechnung_Volt();
			Umrechnung_mbar();
			lcd_writer();
			b_tick10ms=false;
			
		//----------wait approx. 0.8s to avoid heating up SHTxx------------------------------
		for (i=0;i<40000;i++);
		}

	}
	return 0;
}	
*/

